/* Particle generators for the sandbox.
   Copyright (C) 2006 Carl Fredrik Hammar (hammy.lite@gmail.com)

   This file is part of GP2X Sand.

   GP2X Sand is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   GP2X Sand is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details. 

   You should have received a copy of the GNU General Public License along
   with GP2X Sand; if not, write to the Free Software Foundation, Inc., 51
   Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */

#ifndef EMITTER_H
#define EMITTER_H

#include <stdbool.h>

#include "action.h"
#include "sandbox.h"

/* An emitter sits at the top of the sandbox generating new particles each
   frame.  The rate it generates new particles depends on it's width and
   it's flow, the probability of a particle getting generated at each
   pixel directly beneath the emitter.

   Emitters have the following attributes:
   Element:      The type of particle to emit.

   Flow:         The probability that a particle gets generated at each
                 of the pixels directly beneath the emitter. Specified as
                 a value between 0 and 1.

   Position:     At which pixel at the top of the sandbox the emitter
                 begins.

   Width:        Width of the emitter.

   Enabled flag: Whether the emitter should emit new particles. */
struct emitter_str
{
  element_t *element_ptr;
  float flow;
  int position;
  int width; 
  bool enabled_flag;
};

typedef struct emitter_str emitter_t;

/* The three default emitters, that emits the element suggested by there
   name. 
  
   FIXME:  This should not be hard-coded and should probably be moved to the
   sandbox module. */
extern emitter_t emitter_sand;
extern emitter_t emitter_water;
extern emitter_t emitter_oil;
extern emitter_t emitter_salt;

/* Initialize the emitter pointed to by EMITTER_PTR to have the specified
   properties.  EMITTER_PTR mustn't be null, if FLOW isn't between 0 to 1
   it it's set to the closest value that is.*/
void
emitter_init (emitter_t *emitter, element_t* element_ptr, float flow,
	      int position, int width, bool enabled_flag)
  __attribute__ ((nonnull (1)));

/* Initialize every property to zero or false.  EMITTER_PTR mustn't be
   null. */
void
emitter_init_default (emitter_t *emitter)
  __attribute__ ((nonnull));

/* Generate new particles directly beneath the emitter.  Whether a
   particle gets generated at a pixel depends on the flow of the emitter
   which is the probability of this happening. EMITTER_PTR mustn't be
   null. */ 
void
emitter_emit (emitter_t *emitter);

/* Enables the emitter if it was disabled and returns true or disable it
   if enabled and returns false.  EMITTER_PTR mustn't be null. */
bool
emitter_enabled_toggle (emitter_t *emitter);

/* Change the flow of the emitter by STEP and returns the new flow of the
   emitter.  The flow won't get outside of the range 0 to 1.  EMITTER_PTR
   mustn't be null. */
float
emitter_flow_step (emitter_t *emitter, float step);

/** Emitter actions. **/

/* The selected is the emitter of the same element that the pen currently
   paints with, or nothing if there is no corresponding emitter. */

/* Toggle the selected emitter between on and off */
extern action_t *const emitter_selected_action_enabled_toggle_ptr;

/* Increase the flow of the selected emitter by 0.05 up to 1. */
extern action_t *const emitter_selected_action_flow_increase_ptr;

/* Decrease the flow of the selected emitter by 0.05 down to 0. */
extern action_t *const emitter_selected_action_flow_decrease_ptr;

#endif /* EMITTER_H */
