/* Pen for drawing particles to the sandbox.
   Copyright (C) 2006 Carl Fredrik Hammar (hammy.lite@gmail.com)

   This file is part of GP2X Sand.

   GP2X Sand is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   GP2X Sand is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details. 

   You should have received a copy of the GNU General Public License along
   with GP2X Sand; if not, write to the Free Software Foundation, Inc., 51
   Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */

#ifndef PEN_H
#define PEN_H

#include <stdbool.h>

#include "action.h"
#include "sandbox.h"

/** The pen draws a filled circle of new particles centered on the pen's
    current position and as it moves around.
    It has the following attributes:

    Position:      The x and y coordinates of it's current position.
    Radius:        The radius of the circle to draw.
    Paint element: The element to fill the circle with.
    Draw flag:     Only draws if this flag is set. **/
struct pen_str
{
  int x, y;
  int radius;
  element_t *paint_element_ptr;
  bool draw_flag;
};
  
typedef struct pen_str pen_t;

/* The actual pen. */
extern pen_t pen;

/* Initialize the pen with default values. */
void
pen_init_default ();

/* Draw a filled circle of particles centered at the pen's position with
   the radius as stored in the pen.  The element painted is the pen's
   paint_element. */ 
void
pen_draw ();

/* Move the pen horizontally and vertically, DX and DY pixels
   respectively.  Drawing a circle of particles at each pixel moved
   through.  The pen may not leave the sandbox, so if (X, Y) is outside
   the sandbox the new position of the pen is the one closest to (X, Y)
   within the sandbox. */
void
pen_move (int dx, int dy);

/* Set the pen's position to (X, Y).  Drawing circles on each pixel on
   the line between it's old position and it's new position.  The pen may
   not leave the sandbox, so if (X, Y) is outside the sandbox the new
   position of the pen is the one closest to (X, Y) within the sandbox. */
void
pen_warp (int x, int y);

/* Halve the radius of the circle drawn by the pen. */
void
pen_radius_halve ();

/* Double the radius of the circle drawn by the pen. */
void
pen_radius_double ();

/* Set the element the pen draws to PAINT_ELEMENT. */
void
pen_paint_element_set (element_t *paint_element_ptr);

/* Set the flag that enables/disables drawing by the pen. */
void
pen_draw_flag_set (bool draw_flag);

/** Pen actions **/

/* Sets the pen to draw when trigger is down and resets it when trigger is
   up. */
extern action_t *const pen_action_draw_ptr;

/* The following two actions simply wrap pen_radius_double() and
   pen_radius_halve() respectively. */
extern action_t *const pen_action_size_increase_ptr;
extern action_t *const pen_action_size_decrease_ptr;

/* The following two actions cycles through the available elements setting
   the element to pain by the pen to the next and previous element
   respectively. */
extern action_t *const pen_action_paint_element_cycle_forward_ptr;
extern action_t *const pen_action_paint_element_cycle_backward_ptr;

#endif /* not PEN_H */
