/* Model for simulation of falling sand.
   Copyright (C) 2006 Russell Turner,
                      Thomas Ren Sidor,
                      Carl Fredrik Hammar (hammy.lite@gmail.com)

   This file is part of GP2X Sand.

   GP2X Sand is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   GP2X Sand is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details. 

   You should have received a copy of the GNU General Public License along
   with GP2X Sand; if not, write to the Free Software Foundation, Inc., 51
   Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */

#include "sandbox.h"

#include <assert.h>
#include <math.h>
#include <stdlib.h>

#include "utils.h"

/* The sandbox is where all the particles are stored and interact with
   each-other and ti thus the base for this application.  Though the
   analogy to a sandbox doesn't really hold since it isn't palaced
   horizontally, it's such a nice and concise analogy that I decided to
   keep it.

   FIXME: QUIT MAKING EXCUSES AND START COMMENTING ;) */

extern SDL_Surface *screen;

#define SECTORS_HEIGHT (SCREEN_HEIGHT / 16)
#define SECTORS_WIDTH (SCREEN_WIDTH / 16)

#define MOVED (1 << 31)
#define TRANSFORMED (1 << 30)
#define INDEX_MASK (~(MOVED | TRANSFORMED))

element_t elements[ELEMENTS_MAX];

element_t *const element_nothing_ptr = &elements[NOTHING];
element_t *const element_wall_ptr = &elements[WALL];
element_t *const element_sand_ptr = &elements[SAND];
element_t *const element_water_ptr = &elements[WATER];
element_t *const element_oil_ptr = &elements[OIL];
element_t *const element_salt_ptr = &elements[SALT];

/* The +2 is for the edges surrounding the sandbox. */
particle_t sandbox[SCREEN_HEIGHT + 2][SCREEN_WIDTH + 2];
sandbox_sector_t sandbox_sectors[SECTORS_HEIGHT][SECTORS_WIDTH];

action_t sandbox_action_clear;
action_t *const sandbox_action_clear_ptr = &sandbox_action_clear;

particle_t
particle_cons (element_t *element_ptr)
{
  return element_ptr - elements;
}

int
particle_element_index (particle_t particle)
{
  return particle & INDEX_MASK;
}

element_t*
particle_element_get (particle_t particle)
{
  int index = particle_element_index (particle);

  assert (index < ELEMENTS_MAX);
  return &elements[index];
}

uint32_t
particle_type_get (particle_t particle)
{
  return particle & INDEX_MASK;
}

bool
particle_moved_flag_get (particle_t particle)
{
  return particle & MOVED;
}

void
particle_moved_flag_set (particle_t *particle_ptr, bool moved_flag)
{
  assert (particle_ptr);
  
  if (moved_flag)
    *particle_ptr |= MOVED;

  else
    *particle_ptr &= ~MOVED;
}

bool
particle_transformed_flag_get (particle_t particle)
{
  return particle & TRANSFORMED;
}

void
particle_transformed_flag_set (particle_t *particle_ptr,
			       bool transformed_flag)
{
  assert (particle_ptr);
  
  if (transformed_flag)
    *particle_ptr |= TRANSFORMED;

  else
    *particle_ptr &= ~TRANSFORMED;
}

bool
particle_active_flag_get (particle_t particle)
{
  return particle_element_get (particle)->active_flag
    && !particle_transformed_flag_get (particle);
}

bool
particle_fall_flag_get (particle_t particle)
{
  return particle_element_get (particle)->fall_flag;
}

bool
particle_float_flag_get (particle_t particle)
{
  return particle_element_get (particle)->float_flag;
}

bool
particle_mobile_flag_get (particle_t particle)
{
  return particle_element_get (particle)->mobile_flag
    && !particle_moved_flag_get (particle);
}

prob_t
particle_resistance_get (particle_t particle)
{
  return particle_element_get (particle)->resistance;
}

uint32_t
particle_weight_get (particle_t particle)
{
  return particle_element_get (particle)->weight;
}

uint16_t
particle_color_get (particle_t particle)
{
  return particle_element_get (particle)->color;
}

element_t*
sandbox_element_first ()
{
  return &elements[0];
}

element_t*
sandbox_element_last ()
{
  return &elements[ELEMENTS_MAX - 1];
}

element_t*
sandbox_element_next (element_t *element_ptr)
{
  if (element_ptr < &elements[0]
      || element_ptr >= &elements[ELEMENTS_MAX - 1])
    return 0;
  
  else
    return element_ptr + 1;
}

element_t*
sandbox_element_prev (element_t *element_ptr)
{
  if (element_ptr <= &elements[0]
      || element_ptr > &elements[ELEMENTS_MAX - 1])
    return 0;
  
  else
    return element_ptr - 1;
}

void
sandbox_actions_init ();

void
sandbox_action_clear_function (void *data, SDL_Event *event);

void
sandbox_sectors_init ();

// Initializing colors
void sandbox_init()
{
  elements[NOTHING].color = SDL_MapRGB (screen->format, 0, 0, 0);
  elements[NOTHING].weight = 50;
  elements[NOTHING].resistance = prob_cons (0, 0);
  elements[NOTHING].active_flag = false;
  elements[NOTHING].mobile_flag = true;
  elements[NOTHING].fall_flag = false;
  elements[NOTHING].float_flag = true;

  elements[WALL].color = SDL_MapRGB (screen->format, 100, 100, 100);
  elements[WALL].weight = 255;
  elements[WALL].resistance = prob_cons (1, 1);
  elements[WALL].active_flag = false;
  elements[WALL].mobile_flag = false;
  elements[WALL].fall_flag = false;
  elements[WALL].float_flag = true;

  elements[SAND].color = SDL_MapRGB (screen->format, 255, 230, 127);
  elements[SAND].weight = 80;
  elements[SAND].resistance = prob_cons (1, 1);
  elements[SAND].active_flag = false;
  elements[SAND].mobile_flag = true;
  elements[SAND].fall_flag = true;
  elements[SAND].float_flag = false;

  elements[WATER].color = SDL_MapRGB (screen->format, 0, 50, 255);
  elements[WATER].weight = 70;
  elements[WATER].resistance = prob_cons (3, 4);
  elements[WATER].active_flag = false;
  elements[WATER].mobile_flag = true;
  elements[WATER].fall_flag = true;
  elements[WATER].float_flag = false;

  elements[OIL].color = SDL_MapRGB (screen->format, 130, 70, 0);
  elements[OIL].weight = 60;
  elements[OIL].resistance = prob_cons (7, 8);
  elements[OIL].active_flag = false;
  elements[OIL].mobile_flag = true;
  elements[OIL].fall_flag = true;
  elements[OIL].float_flag = false;

  elements[SALT].color = SDL_MapRGB (screen->format, 255, 255, 255);
  elements[SALT].weight = 80;
  elements[SALT].resistance = prob_cons (1, 1);
  elements[SALT].active_flag = true;
  elements[SALT].mobile_flag = true;
  elements[SALT].fall_flag = true;
  elements[SALT].float_flag = false;

  elements[SALT].transformations[WATER].probability = prob_cons (1, 250);
  elements[SALT].transformations[WATER].shuffle_result_flag = false;
  elements[SALT].transformations[WATER].result_primary =
    &elements[SALTWATER]; 
  elements[SALT].transformations[WATER].result_secondary =
    &elements[SALTWATER]; 

  elements[SALTWATER].color = SDL_MapRGB (screen->format, 0, 25, 160);
  elements[SALTWATER].weight = elements[WATER].weight;
  elements[SALTWATER].resistance = elements[WATER].resistance;
  elements[SALTWATER].active_flag = true;
  elements[SALTWATER].mobile_flag = true;
  elements[SALTWATER].fall_flag = true;
  elements[SALTWATER].float_flag = false;

  elements[SALTWATER].transformations[SALTWATER].probability
    = prob_cons (1, 100000);
  elements[SALTWATER].transformations[SALTWATER].shuffle_result_flag
    = true;
  elements[SALTWATER].transformations[SALTWATER].result_primary =
    &elements[SALT]; 
  elements[SALTWATER].transformations[SALTWATER].result_secondary =
    &elements[WATER]; 

  elements[SALTWATER].transformations[WATER].probability
    = prob_cons (1, 25);
  elements[SALTWATER].transformations[WATER].shuffle_result_flag
    = false;
  elements[SALTWATER].transformations[WATER].result_primary =
    &elements[WATER]; 
  elements[SALTWATER].transformations[WATER].result_secondary =
    &elements[SALTWATER]; 

  elements[STEAM].color = SDL_MapRGB (screen->format, 85, 85, 255);
  elements[STEAM].weight = 40;
  elements[STEAM].resistance = prob_cons (0, 0);
  elements[STEAM].active_flag = true;
  elements[STEAM].mobile_flag = true;
  elements[STEAM].fall_flag = false;
  elements[STEAM].float_flag = false;

  for (int i = 0; i < ELEMENTS_MAX; i++)
    {
      elements[STEAM].transformations[i].probability = prob_cons (1, 4000);
      elements[STEAM].transformations[i].shuffle_result_flag = false;
      elements[STEAM].transformations[i].result_primary =
	&elements[WATER];  
      elements[STEAM].transformations[i].result_secondary =
	&elements[i]; 
    }
      
  elements[PLANT].color = SDL_MapRGB (screen->format, 32, 204, 32);
  elements[PLANT].weight = 255;
  elements[PLANT].resistance = prob_cons (1, 1);
  elements[PLANT].active_flag = true;
  elements[PLANT].mobile_flag = false;
  elements[PLANT].fall_flag = false;
  elements[PLANT].float_flag = true;

  elements[PLANT].transformations[WATER].probability = prob_cons (1, 5);
  elements[PLANT].transformations[WATER].shuffle_result_flag = false;
  elements[PLANT].transformations[WATER].result_primary =
    &elements[PLANT]; 
  elements[PLANT].transformations[WATER].result_secondary =
    &elements[PLANT]; 

  elements[FIRE].color = SDL_MapRGB (screen->format, 247, 63, 63);
  elements[FIRE].weight = 30;
  elements[FIRE].resistance = prob_cons (0, 0);
  elements[FIRE].active_flag = true;
  elements[FIRE].mobile_flag = true;
  elements[FIRE].fall_flag = false;
  elements[FIRE].float_flag = false;

  for (int i = 0; i < ELEMENTS_MAX; i++)
    {
      elements[FIRE].transformations[i].probability = prob_cons (1, 100);
      elements[FIRE].transformations[i].shuffle_result_flag = false;
      elements[FIRE].transformations[i].result_primary =
	&elements[NOTHING];  
      elements[FIRE].transformations[i].result_secondary =
	&elements[i]; 
    }
      
  elements[FIRE].transformations[WATER].probability = prob_cons (9, 10);
  elements[FIRE].transformations[WATER].shuffle_result_flag = false;
  elements[FIRE].transformations[WATER].result_primary =
    &elements[NOTHING]; 
  elements[FIRE].transformations[WATER].result_secondary =
    &elements[STEAM]; 

  elements[FIRE].transformations[OIL].probability = prob_cons (3, 4);
  elements[FIRE].transformations[OIL].shuffle_result_flag = false;
  elements[FIRE].transformations[OIL].result_primary =
    &elements[FIRE]; 
  elements[FIRE].transformations[OIL].result_secondary =
    &elements[FIRE]; 

  elements[FIRE].transformations[SALTWATER].probability = prob_cons (9, 10);
  elements[FIRE].transformations[SALTWATER].shuffle_result_flag = true;
  elements[FIRE].transformations[SALTWATER].result_primary =
    &elements[SALT]; 
  elements[FIRE].transformations[SALTWATER].result_secondary =
    &elements[STEAM]; 

  elements[FIRE].transformations[PLANT].probability = prob_cons (1, 1);
  elements[FIRE].transformations[PLANT].shuffle_result_flag = false;
  elements[FIRE].transformations[PLANT].result_primary =
    &elements[FIRE]; 
  elements[FIRE].transformations[PLANT].result_secondary =
    &elements[FIRE]; 

  sandbox_clear ();
  sandbox_sectors_init ();

  sandbox_actions_init ();
}

bool
vect_within_bounds (vect_t v, vect_t bounds)
{
  return v.x >= 0 && v.x < bounds.x && v.y >= 0 && v.y < bounds.y;
}

sandbox_sector_t*
sandbox_sector_containing_particle_get (vect_t position)
{
  vect_t v = { position.x / 16, position.y / 16 };
  vect_t bounds = { SECTORS_WIDTH, SECTORS_HEIGHT };
      
  if (vect_within_bounds (v, bounds))
    return &sandbox_sectors[position.y / 16][position.x / 16];

  else
    return 0;
}

particle_t
sandbox_particle_get (vect_t position)
{
  return sandbox[position.y + 1][position.x + 1];
}

void
sandbox_particle_set (vect_t position, particle_t particle)
{
  sandbox_sector_t *sector_ptr = 
    sandbox_sector_containing_particle_get (position);
  if (sector_ptr != 0)
    sector_ptr->dirty_flag = true;

  vect_t v = { position.x + 1, position.y + 1 };
  sandbox[v.y][v.x] = particle;
}

void
sandbox_particle_init (vect_t position, element_t *element_ptr)
{
  sandbox_particle_set (position, particle_cons (element_ptr));
}
  
void
sandbox_particle_swap (vect_t pp1, vect_t pp2)
{
  particle_t p1 = sandbox_particle_get (pp1);
  particle_t p2 = sandbox_particle_get (pp2);
  
  particle_moved_flag_set (&p1, true);
  particle_moved_flag_set (&p2, true);

  sandbox_particle_set (pp1, p2);
  sandbox_particle_set (pp2, p1);
}
  
void
sandbox_sector_update (sandbox_sector_t *sector_ptr)
{
  if (!sector_ptr->dirty_flag)
    return;
  
  int element_counts[ELEMENTS_MAX] = {0};
  
  particle_t particle_first
    = sandbox_particle_get (sector_ptr->position);
  element_t *element_first_ptr = particle_element_get (particle_first);

  sector_ptr->homogenous_flag = true;
  sector_ptr->stable_flag = true;

  /* Position of the sector's top-left corner. */ 
  vect_t start = { sector_ptr->position.x, sector_ptr->position.y };

  /* Position of the sector's bottom-right corner. */ 
  vect_t end = { start.x + sector_ptr->dimensions.x,
		 start.y + sector_ptr->dimensions.y };

  for (vect_t v = start; v.y < end.y; v.y++)
    {
      for (v.x = start.x; v.x < end.x; v.x++)
	{
	  particle_t particle = sandbox_particle_get (v);
	  element_t *element_ptr = particle_element_get (particle);

	  int element_index = particle_element_index (particle);
	  element_counts[element_index]++;

	  if (element_ptr != element_first_ptr)
	    sector_ptr->homogenous_flag = false;
	  
	  if (element_ptr->active_flag || !element_ptr->float_flag)
	    sector_ptr->stable_flag = false;
	}
    }

  int max_index = 0;
  for (int i = 1; i < LENGTH_OF (element_counts); i++)
    max_index = element_counts[max_index] > element_counts[i]
      ? max_index : i;

  sector_ptr->dominant_element_ptr = &elements[max_index];

  sector_ptr->dirty_flag = false;
}

void
sandbox_sectors_update ()
{
  for (int y = 0; y < SECTORS_HEIGHT; y++)
    for (int x = 0; x < SECTORS_WIDTH; x++)
      sandbox_sector_update (&sandbox_sectors[y][x]);
}

void
sandbox_sectors_get (sandbox_sector_t **sectors_ptr_dst, int *length_dst)
{
  assert (sectors_ptr_dst);
  assert (length_dst);
  
  *sectors_ptr_dst = &sandbox_sectors[0][0];
  *length_dst = SECTORS_WIDTH * SECTORS_HEIGHT;
}

void
sandbox_sector_init (sandbox_sector_t *sector_ptr,
		     int x, int y, int width, int height)
{
  sector_ptr->position.x = x;
  sector_ptr->position.y = y;
  sector_ptr->dimensions.x = width;
  sector_ptr->dimensions.y = height;
  
  sector_ptr->dirty_flag = true;
  sandbox_sector_update (sector_ptr);
}

void
sandbox_sectors_init ()
{
  int sector_width = SCREEN_WIDTH / SECTORS_WIDTH;
  int sector_height = SCREEN_HEIGHT / SECTORS_HEIGHT;

  for (int y = 0; y < SECTORS_HEIGHT; y++)
    for (int x = 0; x < SECTORS_WIDTH; x++)
      sandbox_sector_init (&sandbox_sectors[y][x],
			   x * sector_width, y * sector_height,
			   sector_width, sector_height);
}

bool
particle_swap_test (particle_t upper, particle_t lower, int dy)
{
  int upper_weight = particle_weight_get (upper) * dy;
  bool upper_moveable = particle_mobile_flag_get (upper);

  int lower_weight = particle_weight_get (lower) * dy;
  bool lower_moveable = particle_mobile_flag_get (lower);

  prob_t resistance = particle_resistance_get (lower);

  return upper_moveable && lower_moveable && upper_weight > lower_weight
    && !prob_boolean (resistance);
}

vect_t
pull_get (vect_t particle_position)
{
  vect_t same = particle_position;
  
  if (prob_boolean (prob_cons (1, 13)))
    return same; // This makes it fall unevenly

  particle_t home = sandbox_particle_get (same);

  int dy = particle_fall_flag_get (home) ? 1 : -1;
  
  vect_t below = { same.x, same.y + dy };
  
  particle_t below_particle = sandbox_particle_get (below);

  /* 7/8 probability to swap with particle below if possible. */
  if (particle_swap_test (home, below_particle, dy)
      && prob_boolean (prob_cons (7, 8)))
    return below;

  /* Randomly select right or left first. */
  int dx = random_boolean () ? -1 : 1;
	
  vect_t firstdown = { same.x + dx, same.y + dy };
  particle_t firstdown_particle
    = sandbox_particle_get (firstdown);

  if (particle_swap_test (home, firstdown_particle, dy))
    return firstdown;
  
  vect_t seconddown = { same.x - dx, same.y + dy };
  particle_t seconddown_particle
    = sandbox_particle_get (seconddown);

  if (particle_swap_test (home, seconddown_particle, dy))
    return seconddown;

  if (prob_boolean (prob_cons (9, 10)))
    return same;
		   
  vect_t first = { same.x + dx, same.y };
  particle_t first_particle
    = sandbox_particle_get (first);

  if (particle_swap_test (home, first_particle, dy))
    return first;

  vect_t second = { same.x - dx, same.y };
  particle_t second_particle
    = sandbox_particle_get (second);

  if (particle_swap_test (home, second_particle, dy))
    return second;

  return same;
}

void
particles_transform (vect_t pp1, vect_t pp2)
{
  particle_t p1 = sandbox_particle_get (pp1);
  particle_t p2 = sandbox_particle_get (pp2);
  
  element_t *element1_ptr = particle_element_get (p1);
  element_t *element2_ptr = particle_element_get (p2);

  int i = element2_ptr - elements;
  transformation_t *trans_ptr = &element1_ptr->transformations[i];
  
  if (prob_boolean (trans_ptr->probability))
    {
      element_t *r1 = trans_ptr->result_primary;
      element_t *r2 = trans_ptr->result_secondary;
      
      if (r1 == 0 || r2 == 0)
	return;
      
      if (trans_ptr->shuffle_result_flag && random_boolean ())
	{
	  element_t *temp = r1;
	  r1 = r2;
	  r2 = temp;
	}
      
      particle_t rp1 = particle_cons (r1);
      particle_transformed_flag_set (&rp1, true);
      particle_moved_flag_set (&rp1, particle_moved_flag_get (p1));
      sandbox_particle_set (pp1, rp1);

      particle_t rp2 = particle_cons (r2);
      particle_transformed_flag_set (&rp2, true);
      particle_moved_flag_set (&rp2, particle_moved_flag_get (p2));
      sandbox_particle_set (pp2, rp2);
    }
}

void
neighbourhood_transform (vect_t particle_position)
{
  static vect_t normal_directions[] = { { 0, 1 },    /* Up. */
					{ 1, 0 },    /* Right. */
					{ 0, -1 },   /* Down. */
					{ -1, 0 } }; /* Left. */

  static vect_t diagonal_directions[] = { { 1, 1 },    /* Up-right. */
					  { 1, -1 },   /* Down-right. */
					  { -1, -1 },  /* Down-left. */
					  { -1, 1 } }; /* Up-left. */

  vect_t *directions;
  
  if (random_boolean ())
    directions = normal_directions;

  else
    directions = diagonal_directions;

  /* Shuffle directions. */
  uint32_t i = rand () % LENGTH_OF (normal_directions);
  uint32_t j = rand () % LENGTH_OF (normal_directions);
  SWAP (directions[i], directions[j]);

  i = rand () % LENGTH_OF (normal_directions);
  j = rand () % LENGTH_OF (normal_directions);
  SWAP (directions[i], directions[j]);

  for (int i = 0; i < LENGTH_OF (normal_directions); i++)
    particles_transform (particle_position, vadd (particle_position,
						  directions[i]));
}
  
void
neighbourhood_update (vect_t particle_position)
{
  vect_t same = particle_position;
  particle_t home = sandbox_particle_get (same);
  
  if (particle_active_flag_get (home)) 
    neighbourhood_transform (same);
  
  if (!particle_float_flag_get (home) && particle_mobile_flag_get (home))
    {
      vect_t pull = pull_get (same);
      
      if (! veql (pull, same))
	sandbox_particle_swap (pull, same);
    }
}

void
sandbox_edges_clear ()
{
  /* Clear top line. */
  for (vect_t v = { -1, -1 }; v.x < SCREEN_WIDTH + 1; v.x++)
    sandbox_particle_init (v, element_nothing_ptr);
  
  /* Clear bottom line. */
  for (vect_t v = { -1, SCREEN_HEIGHT }; v.x < SCREEN_WIDTH + 1; v.x++)
    sandbox_particle_init (v, element_nothing_ptr);
  
  /* Clear left-most column. */
  for (vect_t v = { -1, -1 }; v.y < SCREEN_HEIGHT + 1; v.y++)
    sandbox_particle_init (v, element_nothing_ptr);

  /* Clear right-most column. */
  for (vect_t v = { SCREEN_WIDTH, -1 }; v.y < SCREEN_HEIGHT + 1; v.y++)
    sandbox_particle_init (v, element_nothing_ptr);
}

void
sandbox_update ()
{
  sandbox_edges_clear ();
  
  for(vect_t v = { 0, SCREEN_HEIGHT - 1}; v.y >= 0; v.y--)
    {
      /* Loop in other direction half of the time (randomly). To prevent
	 any horizontal bias. */
      if (random_boolean ())
	{
	  for (v.x = 0; v.x < SCREEN_WIDTH;)
	    {
	      sandbox_sector_t *sector_ptr =
		sandbox_sector_containing_particle_get (v);
	      
	      int x_start = sector_ptr->position.x;
	      int x_end = x_start + sector_ptr->dimensions.x;
		  
	      if (sector_ptr->dirty_flag || !sector_ptr->stable_flag)
		{
		  for (v.x = x_start; v.x < x_end; v.x++)
		    neighbourhood_update (v);
		}

	      else
		v.x = x_end;
	    }
	}
      else
	{
	  for (v.x = SCREEN_WIDTH - 1; v.x >= 0;)
	    {
	      sandbox_sector_t *sector_ptr =
		sandbox_sector_containing_particle_get (v);

	      int x_end = sector_ptr->position.x - 1;
	      int x_start = x_end + sector_ptr->dimensions.x;
		  
	      if (sector_ptr->dirty_flag || !sector_ptr->stable_flag)
		{
		  for (v.x = x_start; v.x > x_end; v.x--)
		    neighbourhood_update (v);
		}

	      else
		v.x = x_end;
	    }
	}
    }
  
  /* Reset particles' moved and transformed flags. */

  for (int y = 0; y < SCREEN_HEIGHT; y++)
    {
      for (int x = 0; x < SCREEN_WIDTH; x++)
	{
	  vect_t v = vcons (x, y);
	  particle_t particle = sandbox_particle_get (v);

	  particle_moved_flag_set (&particle, false);
	  particle_transformed_flag_set (&particle, false);
	  sandbox_particle_set (v, particle);
	}
    }

  sandbox_sectors_update ();
}

void
sandbox_clear ()
{
  for (vect_t v = { 0, 0 }; v.y < SCREEN_HEIGHT; v.y++)
    for (v.x = 0; v.x < SCREEN_WIDTH; v.x++)
      sandbox_particle_init (v, element_nothing_ptr);
}

void
sandbox_actions_init ()
{
  sandbox_action_clear.function = sandbox_action_clear_function;
}

void
sandbox_action_clear_function (void *data, SDL_Event *event)
{
  if (event_trigger_state (event) == TRIGGER_DOWN)
    sandbox_clear ();
}
